"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const winston = require("winston");
const moment = require("moment");
const sql = require("mssql");
const fs = require("fs");
const path = require("path");
const delay = require("delay");
const readChunk = require("read-chunk");
const zlib = require("zlib");
const dropbox_gateway_1 = require("./dropbox-gateway");
const argv = require('yargs')
    .usage('Usage: $0 [options]')
    .demand('fileNamePrefix', 'Filename Prefix')
    .demand('filePath', 'File Path')
    .demand('databaseName', 'Database Name')
    .demand('databaseUser', 'Database User')
    .demand('databasePassword', 'Database Password')
    .demand('databaseHost', 'Database Host')
    .demand('accessToken', 'Dropbox Access Token')
    .argv;
const fileNamePrefix = argv.fileNamePrefix;
const filePath = argv.filePath;
const databaseName = argv.databaseName;
const databaseUser = argv.databaseUser;
const databaseHost = argv.databaseHost;
const databasePassword = argv.databasePassword;
const accessToken = argv.accessToken;
app().catch((err) => {
    winston.error(err);
});
function app() {
    return __awaiter(this, void 0, void 0, function* () {
        const fileName = `${fileNamePrefix}_${moment().format('MM-DD-YYYY-HH-mm-ss')}.Bak`;
        const compressedFileName = `${fileName}.gz`;
        yield backupDatabase(databaseHost, databaseUser, databasePassword, databaseName, path.join(filePath, fileName));
        yield compressFile(path.join(filePath, fileName), path.join(filePath, compressedFileName));
        yield uploadFile(path.join(filePath, compressedFileName), accessToken);
    });
}
function backupDatabase(host, user, password, database, filename) {
    return __awaiter(this, void 0, void 0, function* () {
        const sqlConfig = {
            user: user,
            password: password,
            server: host,
            database: database
        };
        const pool = yield sql.connect(sqlConfig);
        const sqlResult = yield pool.request().query(`BACKUP DATABASE ${database} TO DISK = '${filename}'`);
        pool.close();
        return;
    });
}
function compressFile(inputFilename, outputFilename) {
    return new Promise((resolve, reject) => {
        const gzip = zlib.createGzip();
        const inputFile = fs.createReadStream(inputFilename);
        const outputFile = fs.createWriteStream(outputFilename);
        inputFile.pipe(gzip).pipe(outputFile);
        outputFile.on('finish', () => {
            inputFile.close();
            inputFile.destroy();
            outputFile.close();
            resolve();
        });
    });
}
function uploadFile(filename, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const dropBoxGateway = new dropbox_gateway_1.DropBoxGateway();
        const fileSize = fs.statSync(filename).size;
        const chunkSize = 100000;
        const sessionId = yield dropBoxGateway.startSession(accessToken);
        for (let i = 0; i < fileSize; i = i + chunkSize) {
            const buffer = readChunk.sync(filename, i, chunkSize);
            for (let j = 0; j < 5; j++) {
                try {
                    yield dropBoxGateway.appendSession(token, sessionId, i, buffer);
                    break;
                }
                catch (err) {
                    winston.error(err);
                    yield delay(1000);
                }
            }
            winston.info(`${i} / ${fileSize}`);
        }
        yield dropBoxGateway.endSession(accessToken, sessionId, `/${path.basename(filename)}`, fileSize);
        return;
    });
}
//# sourceMappingURL=app.js.map