"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const rp = require("request-promise");
class DropBoxGateway {
    startSession(accessToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = {
                "close": false
            };
            const json = yield rp({
                method: 'POST',
                uri: 'https://content.dropboxapi.com/2/files/upload_session/start',
                headers: {
                    Authorization: `Bearer ${accessToken}`,
                    'Dropbox-API-Arg': JSON.stringify(parameters),
                    'Content-Type': 'application/octet-stream'
                },
                json: true
            });
            return json.session_id;
        });
    }
    appendSession(accessToken, sessionId, offset, buffer) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = {
                "cursor": {
                    "session_id": sessionId,
                    "offset": offset
                },
                "close": false
            };
            yield rp({
                method: 'POST',
                uri: 'https://content.dropboxapi.com/2/files/upload_session/append_v2',
                headers: {
                    Authorization: `Bearer ${accessToken}`,
                    'Dropbox-API-Arg': JSON.stringify(parameters),
                    'Content-Type': 'application/octet-stream'
                },
                body: buffer
            });
            return true;
        });
    }
    endSession(accessToken, sessionId, filename, fileSize) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = {
                "cursor": {
                    "session_id": sessionId,
                    "offset": fileSize
                },
                "commit": {
                    "path": filename,
                    "mode": "add",
                    "autorename": true,
                    "mute": false
                }
            };
            yield rp({
                method: 'POST',
                uri: 'https://content.dropboxapi.com/2/files/upload_session/finish',
                headers: {
                    Authorization: `Bearer ${accessToken}`,
                    'Dropbox-API-Arg': JSON.stringify(parameters),
                    'Content-Type': 'application/octet-stream'
                }
            });
            return true;
        });
    }
}
exports.DropBoxGateway = DropBoxGateway;
//# sourceMappingURL=dropbox-gateway.js.map